import { SyncAction, SyncParamAction } from '@/shared/foundation';
import { reduceIndexed } from '@u/funcs';
import { append, dropLast, equals, length } from 'ramda';
import { createStoreHook } from './store_creator';

export interface BreadcrumbRecord {
  label: string;
  link?: string;
}

/**
 * 用于记录面包屑导航的存储。
 */
interface BreadcrumbStore {
  /**
   * 目前记录的各级面包屑。
   */
  breadcrumbs: BreadcrumbRecord[];
}

type Actions = {
  /**
   * 向当前面包屑记录中推送一条新的记录。
   */
  push: SyncParamAction<BreadcrumbRecord>;
  /**
   * 从当前面包屑记录中弹出最后一条记录。
   */
  pop: SyncAction;
  /**
   * 删除当前全部面包屑记录，替换成新的记录。
   */
  replace: SyncParamAction<BreadcrumbRecord>;
};

const initialState: BreadcrumbStore = {
  breadcrumbs: []
};

export const useBreadcrumbStore = createStoreHook<BreadcrumbStore & Actions>(set => ({
  ...initialState,
  push: record =>
    set(st => {
      st.breadcrumbs = append(record, st.breadcrumbs);
    }),
  pop: () =>
    set(st => {
      st.breadcrumbs = dropLast(1, st.breadcrumbs);
    }),
  replace: record => set({ breadcrumbs: [record] })
}));

/**
 * 用于生成一套最后一位面包屑不带链接的面包屑集合。
 */
export const breadcrumbsSelector: (state: BreadcrumbStore) => BreadcrumbRecord[] = state => {
  return reduceIndexed(
    (acc, elem, index, list) => {
      if (equals(index, length(list) - 1)) {
        return append({ label: elem.label }, acc);
      }
      return append(elem, acc);
    },
    [],
    state.breadcrumbs
  );
};
